@@ -1,5 +1,5 @@
 #
-# $Id: IMDB.pm,v 1.53 2007/01/12 20:08:47 jveldhuis Exp $
+# $Id: IMDB.pm,v 1.56 2008/09/25 18:01:01 jveldhuis Exp $
 #
 # The IMDB file contains two packages:
 # 1. XMLTV::IMDB::Cruncher package which parses and manages IMDB "lists" files
@@ -33,7 +33,16 @@ use strict;
 
 package XMLTV::IMDB;
 
-our $VERSION = '0.6';
+#
+# HISTORY
+# .6 = what was here for the longest time
+# .7 = fixed file size est calculations
+#    = moviedb.info now includes _file_size_uncompressed values for each downloaded file
+# .8 = updated file size est calculations
+#    = moviedb.dat directors and actors list no longer include repeated names (which mostly
+#      occured in episodic tv programs (reported by Alexy Khrabrov)
+#
+our $VERSION = '0.8';
 
 sub new
 {
@@ -236,11 +245,17 @@ sub basicVerificationOfIndexes($)
 	return("basic verification of indexes failed\n".
 	       "movie details didn't show Bruce Campbell as the main actor in movie \"$title, $year\" (id=$id)\n");
     }
-    if ( $res->{genres}[0] ne "Action" ||
-	 $res->{genres}[1] ne "Adventure" ||
-	 $res->{genres}[2] ne "Comedy" ||
-	 $res->{genres}[3] ne "Fantasy" ||
-	 $res->{genres}[4] ne "Horror" ) {
+    my $matches=0;
+    for (@{$res->{genres}}) {
+	if ( $_ eq "Action" ||
+	     $_ eq "Comedy" ||
+	     $_ eq "Fantasy" ||
+	     $_ eq "Horror" ||
+	     $_ eq "Romance" ) {
+	    $matches++;
+	}
+    }
+    if ( $matches == 0 ) {
 	$self->closeMovieIndex();
 	return("basic verification of indexes failed\n".
 	       "movie details didn't show genres correctly for movie \"$title, $year\" (id=$id)\n");
@@ -1877,9 +1892,9 @@ sub dbinfoSave($)
     return(0);
 }
 
-sub dbinfoCalcEstimate($$$)
+sub dbinfoGetFileSize($$)
 {
-    my ($self, $key, $estimateSizePerEntry)=@_;
+    my ($self, $key)=@_;
     
     if ( !defined($self->{imdbListFiles}->{$key}) ) {
 	die ("invalid call");
@@ -1890,7 +1905,7 @@ sub dbinfoCalcEstimate($$$)
     if ( $self->{imdbListFiles}->{$key}=~m/.gz$/) {
 	if ( open(my $fd, "gzip -l ".$self->{imdbListFiles}->{$key}."|") ) {
 	    # if parse fails, then defalt to wild ass guess of compression of 65%
-	    $fileSize=int($fileSize*100)/(100-65);
+	    $fileSize=int(($fileSize*100)/(100-65));
 
 	    while(<$fd>) {
 		if ( m/^\s*\d+\s+(\d+)/ ) {
@@ -1901,17 +1916,36 @@ sub dbinfoCalcEstimate($$$)
 	}
 	else {
 	    # wild ass guess of compression of 65%
-	    $fileSize=int($fileSize*100)/(100-65);
+	    $fileSize=int(($fileSize*100)/(100-65));
 	}
     }
-    my $countEstimate=int($fileSize/$estimateSizePerEntry);
+    return($fileSize);
+}
 
+sub dbinfoCalcEstimate($$$)
+{
+    my ($self, $key, $estimateSizePerEntry)=@_;
+
+    my $fileSize=$self->dbinfoGetFileSize($key);
+
+    my $countEstimate=int($fileSize/$estimateSizePerEntry);
+    
     $self->dbinfoAdd($key."_list_file", $self->{imdbListFiles}->{$key});
-    $self->dbinfoAdd($key."_list_file_size", $fileSize);
+    $self->dbinfoAdd($key."_list_file_size", int(-s "$self->{imdbListFiles}->{$key}"));
+    $self->dbinfoAdd($key."_list_file_size_uncompressed", $fileSize);
     $self->dbinfoAdd($key."_list_count_estimate", $countEstimate);
     return($countEstimate);
 }
 
+sub dbinfoCalcBytesPerEntry($$$)
+{
+    my ($self, $key, $calcActualForThisNumber)=@_;
+
+    my $fileSize=$self->dbinfoGetFileSize($key);
+    
+    return(int($fileSize/$calcActualForThisNumber));
+}
+
 sub invokeStage($$)
 {
     my ($self, $stage)=@_;
@@ -1928,8 +1962,9 @@ sub invokeStage($$)
 	    }
 	    return(1);
 	}
-	elsif ( abs($num - $countEstimate) > $countEstimate*.05 ) {
-	    $self->status("ARG estimate of $countEstimate for movies needs updating, found $num");
+	elsif ( abs($num - $countEstimate) > $countEstimate*.10 ) {
+	    my $better=$self->dbinfoCalcBytesPerEntry("movies", $num);
+	    $self->status("ARG estimate of $countEstimate for movies needs updating, found $num ($better bytes/entry)");
 	}
 	$self->dbinfoAdd("db_stat_movie_count", "$num");
 
@@ -1969,7 +2004,7 @@ sub invokeStage($$)
     elsif ( $stage == 2 ) {
 	$self->status("parsing Directors list for stage $stage..");
 
-	my $countEstimate=$self->dbinfoCalcEstimate("directors", 184);
+	my $countEstimate=$self->dbinfoCalcEstimate("directors", 225);
 
 	my $num=$self->readCastOrDirectors("Directors", $countEstimate, "$self->{imdbListFiles}->{directors}");
 	if ( $num < 0 ) {
@@ -1978,8 +2013,9 @@ sub invokeStage($$)
 	    }
 	    return(1);
 	}
-	elsif ( abs($num - $countEstimate) > $countEstimate*.05 ) {
-	    $self->status("ARG estimate of $countEstimate for directors needs updating, found $num");
+	elsif ( abs($num - $countEstimate) > $countEstimate*.10 ) {
+	    my $better=$self->dbinfoCalcBytesPerEntry("directors", $num);
+	    $self->status("ARG estimate of $countEstimate for directors needs updating, found $num ($better bytes/entry)");
 	}
 	$self->dbinfoAdd("db_stat_director_count", "$num");
 
@@ -2035,7 +2071,7 @@ sub invokeStage($$)
 	$self->status("parsing Actors list for stage $stage..");
 
 	#print "re-reading movies into memory for reverse lookup..\n";
-	my $countEstimate=$self->dbinfoCalcEstimate("actors", 349);
+	my $countEstimate=$self->dbinfoCalcEstimate("actors", 415);
 
 	my $num=$self->readCastOrDirectors("Actors", $countEstimate, "$self->{imdbListFiles}->{actors}");
 	if ( $num < 0 ) {
@@ -2044,8 +2080,9 @@ sub invokeStage($$)
 	    }
 	    return(1);
 	}
-	elsif ( abs($num - $countEstimate) > $countEstimate*.05 ) {
-	    $self->status("ARG estimate of $countEstimate for actors needs updating, found $num");
+	elsif ( abs($num - $countEstimate) > $countEstimate*.10 ) {
+	    my $better=$self->dbinfoCalcBytesPerEntry("actors", $num);
+	    $self->status("ARG estimate of $countEstimate for actors needs updating, found $num ($better bytes/entry)");
 	}
 	$self->dbinfoAdd("db_stat_actor_count", "$num");
 
@@ -2085,7 +2122,7 @@ sub invokeStage($$)
     elsif ( $stage == 4 ) {
 	$self->status("parsing Actresses list for stage $stage..");
 
-	my $countEstimate=$self->dbinfoCalcEstimate("actresses", 311);
+	my $countEstimate=$self->dbinfoCalcEstimate("actresses", 382);
 	my $num=$self->readCastOrDirectors("Actresses", $countEstimate, "$self->{imdbListFiles}->{actresses}");
 	if ( $num < 0 ) {
 	    if ( $num == -2 ) {
@@ -2093,8 +2130,9 @@ sub invokeStage($$)
 	    }
 	    return(1);
 	}
-	elsif ( abs($num - $countEstimate) > $countEstimate*.05 ) {
-	    $self->status("ARG estimate of $countEstimate for actresses needs updating, found $num");
+	elsif ( abs($num - $countEstimate) > $countEstimate*.10 ) {
+	    my $better=$self->dbinfoCalcBytesPerEntry("actresses", $num);
+	    $self->status("ARG estimate of $countEstimate for actresses needs updating, found $num ($better bytes/entry)");
 	}
 	$self->dbinfoAdd("db_stat_actress_count", "$num");
 
@@ -2142,8 +2180,9 @@ sub invokeStage($$)
 	    }
 	    return(1);
 	}
-	elsif ( abs($num - $countEstimate) > $countEstimate*.05 ) {
-	    $self->status("ARG estimate of $countEstimate for genres needs updating, found $num");
+	elsif ( abs($num - $countEstimate) > $countEstimate*.10 ) {
+	    my $better=$self->dbinfoCalcBytesPerEntry("genres", $num);
+	    $self->status("ARG estimate of $countEstimate for genres needs updating, found $num ($better bytes/entry)");
 	}
 	$self->dbinfoAdd("db_stat_genres_count", "$num");
 
@@ -2191,8 +2230,9 @@ sub invokeStage($$)
 	    }
 	    return(1);
 	}
-	elsif ( abs($num - $countEstimate) > $countEstimate*.05 ) {
-	    $self->status("ARG estimate of $countEstimate for ratings needs updating, found $num");
+	elsif ( abs($num - $countEstimate) > $countEstimate*.10 ) {
+	    my $better=$self->dbinfoCalcBytesPerEntry("ratings", $num);
+	    $self->status("ARG estimate of $countEstimate for ratings needs updating, found $num ($better bytes/entry)");
 	}
 	$self->dbinfoAdd("db_stat_ratings_count", "$num");
 
@@ -2662,9 +2702,13 @@ sub invokeStage($$)
 		    $details.="<>";
 		}
 		else {
-		    # sort directors by last name
+		    # sort directors by last name, removing duplicates
+		    my $last='';
 		    for my $name (sort {$a cmp $b} split('\|', $directors)) {
-			$details.="$name|";
+			if ( $name ne $last ) {
+			    $details.="$name|";
+			    $last=$name;
+			}
 		    }
 		    $details=~s/\|$//o;
 		}
@@ -2676,8 +2720,9 @@ sub invokeStage($$)
 		else {
 		    $details.=$tab;
 
-		    # sort actors by billing
+		    # sort actors by billing, removing repeated entries
 		    # be warned, two actors may have the same billing level
+		    my $last='';
 		    for my $c (sort {$a cmp $b} split('\|', $actors)) {
 			my ($billing, $name)=split(':', $c);
 			# remove Host/Narrators from end
@@ -2685,7 +2730,10 @@ sub invokeStage($$)
 			$name=~s/\s\([IVX]+\)\[/\[/o;
 			$name=~s/\s\([IVX]+\)$//o;
 			
-			$details.="$name|";
+			if ( $name ne $last ) {
+			    $details.="$name|";
+			    $last=$name;
+			}
 			#print "      $c: split gives'$billing' and '$name'\n";
 		    }
 		    $details=~s/\|$//o;
