@@ -2,7 +2,6 @@ package XMLTV::Options;
 
 use strict;
 use warnings;
-use vars qw($fd);
 
 BEGIN {
     use Exporter   ();
@@ -17,7 +16,7 @@ our @EXPORT_OK;
 
 =head1 NAME
 
-XMLTV::Options
+XMLTV::Options - Command-line parsing for XMLTV grabbers
 
 =head1 DESCRIPTION
 
@@ -35,8 +34,6 @@ use XMLTV::Configure qw/LoadConfig Configure SelectChannelsStage/;
 
 use Getopt::Long;
 use Carp qw/croak/;
-use IO::Wrap qw/wraphandle/;
-use IO::Scalar;
 
 my %cap_options = (
 		   all => [qw/
@@ -128,7 +125,7 @@ The entries in the hash configure the behaviour of ParseOptions.
 
   my( $opt, $conf ) = ParseOptions( {
     grabber_name => 'tv_grab_test',
-    version => '$Id: Options.pm,v 1.18 2010/03/31 06:27:42 rmeden Exp $',
+    version => '$Id: Options.pm,v 1.21 2010/10/01 17:49:30 dekarl Exp $',
     description => 'Sweden (tv.swedb.se)',
     capabilities => [qw/baseline manualconfig apiconfig lineups/],
     stage_sub => \&config_stage,
@@ -302,7 +299,7 @@ Optional. A value to return when the grabber is called with the
 
   my( $opt, $conf ) = ParseOptions( {
     grabber_name => 'tv_grab_test',
-    version => '$Id: Options.pm,v 1.18 2010/03/31 06:27:42 rmeden Exp $',
+    version => '$Id: Options.pm,v 1.21 2010/10/01 17:49:30 dekarl Exp $',
     description => 'Sweden (tv.swedb.se)',
     capabilities => [qw/baseline manualconfig apiconfig preferredmethod/],
     stage_sub => \&config_stage,
@@ -438,14 +435,15 @@ sub ParseOptions
 
     if( defined( $opt->{output} ) )
     {
-	if( not open( $fd, "> $opt->{output}" ) )
+	# Redirect STDOUT to the file.
+	if( not open( STDOUT, "> $opt->{output}" ) )
 	{
 	    print STDERR "Cannot write to $opt->{output}.";
 	    exit 1;
 	}
 	
-	# Redirect STDOUT to the file.
-	select( $fd );
+        # Redirect default output to STDOUT
+	select( STDOUT );
     }
 
     if( $opt->{configure} )
