@@ -1,14 +1,17 @@
-# Miscellaneous timezone routines.  The code in Europe_TZ.pm builds on
-# these for handling European summer time conventions.  This should
+# Miscellaneous timezone routines.  The code in DST.pm builds on
+# these for handling summer time conventions.  This should
 # probably be moved into Date::Manip somehow.
 #
+# $Id: TZ.pm,v 1.17 2010/03/28 04:04:36 crispygoth Exp $
+#
 
 package XMLTV::TZ;
 use Carp;
 use Date::Manip; # no Date_Init(), that can be done by the app
+use XMLTV::Date;
 # Won't Memoize, you can do that yourself.
 use base 'Exporter'; our @EXPORT_OK;
-@EXPORT_OK = qw(gettz ParseDate_PreservingTZ tz_to_num parse_local_date);
+@EXPORT_OK = qw(gettz ParseDate_PreservingTZ tz_to_num parse_local_date offset_to_gmt);
 
 # Use Log::TraceMessages if installed.
 BEGIN {
@@ -53,24 +56,45 @@ sub ParseDate_PreservingTZ($) {
     croak 'usage: ParseDate_PreservingTZ(unparsed date string)'
       if @_ != 1;
     my $u = shift;
-    my $p = ParseDate($u); return undef if not defined $p;
+    my $p = ParseDate($u);
+    die "cannot parse $u" if not $p;
     my $tz = gettz($u) || 'UTC';
+
+    my $ltz=Date_TimeZone(); # avoid bug in Date::Manip 6.05
+    $ltz=$tz if $ltz eq "1"; # if Date_TimeZone returns a bad value, use something ok
+
 #    print STDERR "date $u parsed to $p (timezone read as $tz)\n";
-    $p = Date_ConvTZ($p, undef, $tz);
+    $p = Date_ConvTZ($p, offset_to_gmt($ltz), offset_to_gmt($tz));
 #    print STDERR "...converted to $p\n";
     return $p;
 }
 
+# Date::Manip version 6 has problems with +nnnn offsets
+# It seems to treat +0000 as equivalent to "Europe/London", meaning that during DST +0000 actually refers to GMT + 1 hour.
+# However, a timezone of etc/gmt+1 will always work.
+# Using this function on arguments to Date_ConvTZ should work around this bug.
+sub offset_to_gmt($) {
+	my $tz = shift;
+
+	return $tz unless $tz =~ /^([+-])0(\d)00/;
+
+	if ($Date::Manip::VERSION >= 6) {
+		if ($2 == 0) {
+			$tz = "etc/gmt";
+		} else {
+			$tz = "etc/gmt$1$2";
+		}	
+	}
+	return $tz;
+}
+		
 
 # tz_to_num()
 #
 # Turn a timezone string into a numeric form.  For example turns 'CET'
 # into '+0100'.  If the timezone is already numeric it's unchanged.
 #
-# Returns undef if the timezone is not recognized.  (OK, throwing an
-# exception would probably make more sense, but Date::Manip has its
-# peculiar interface style where you have to manually check the result
-# of every call and we might as well fit into that.)
+# Throws an exception if the timezone is not recognized.
 #
 sub tz_to_num( $ ) {
     my $tz = shift;
@@ -85,10 +109,10 @@ sub tz_to_num( $ ) {
     # To convert to a number we parse a date with this timezone and
     # then compare against the same date with UTC.
     #
-    my $date_str = '2000-01-01 00:00:00'; # arbitrary
-    my $base = ParseDate("$date_str UTC"); die if not defined $base;
+    my $date_str = '2000-08-01 00:00:00'; # arbitrary
+    my $base = parse_date("$date_str UTC");
     t "parsed '$date_str UTC' as $base";
-    my $d = ParseDate("$date_str $tz"); return undef if not defined $d;
+    my $d = parse_date("$date_str $tz");
     t "parsed '$date_str $tz' as $base";
     my $err;
     my $delta = DateCalc($d, $base, \$err);
@@ -97,9 +121,8 @@ sub tz_to_num( $ ) {
     # A timezone difference must be less than one day, and must be a
     # whole number of minutes.
     #
-    $delta =~ /^([+-])0:0:0:0:(\d\d?):(\d\d?):0$/ or die "bad delta $delta";
-    t "turned timezone $tz into delta $1 $2 $3";
-    return sprintf('%s%02d%02d', $1, $2, $3);
+    my @df = Delta_Format($delta, 0, "%hv", "%Mv");
+    return sprintf('%s%02d%02d', ($df[0] < 0) ? '-' : '+', abs($df[0]), $df[1]);
 }
 
 
@@ -125,7 +148,7 @@ sub tz_to_num( $ ) {
 # NB, use this function _before_ changing the default timezone to UTC,
 # if you want to parse some dates in the user's local timezone!
 #
-# Like ParseDate(), returns undef on error.
+# Throws an exception on error.
 #
 sub parse_local_date( $ ) {
     my $d = shift;
@@ -133,8 +156,8 @@ sub parse_local_date( $ ) {
     t 'parse_local_date() parsing: ' . d $d;
     my $pd = ParseDate($d);
     t 'ParseDate() returned: ' . d $pd;
-    return undef if not defined $pd;
-    my $r = Date_ConvTZ($pd, Date_TimeZone(), 'UTC');
+    die "cannot parse date $d" if not $pd;
+    my $r = Date_ConvTZ($pd, offset_to_gmt(Date_TimeZone()), 'UTC');
     t 'converted into UTC: ' . d $r;
     return $r;
 }
